/******************************************************************************
 *
 * smap.c - a simple SIP scanner
 *
 * smap is a mashup of nmap and sipsak, thus the ugly name
 *
 * (c) 2006 by Hendrik Scholz <hscholz@raisdorf.net>
 * http://www.wormulon.net/
 *
 * $Id: smap.c,v 1.20 2006/08/27 09:46:13 hscholz Exp $
 *****************************************************************************/
#ifndef _SMAP_C
#define _SMAP_C
#include "config.h"

void usage(int rc, char **argv) {
	fprintf(stderr, "usage: smap [ Options ] <ip | ip/mask | host>\n\n"
		"            -h: this help\n"
		"            -d: increase debugging\n"
		"            -o: enable fingerprinting\n"
		"            -O: enable more verbose fingerprinting\n"
						/* shameless copy from nmap ;) */
		"            -l: fingerprint learning mode\n"
		"            -t: TCP transport\n"
		"            -u: UDP transport (default\n"
		"           -P0: Treat all hosts as online - skip host discovery\n"
		"     -p <port>: destination port\n"
		"   -D <domain>: SIP domain to use without leading sip:\n"
		"  -w <timeout>: timeout in msec\n\n");
	exit ((rc > 0) ? rc : 0);
}

/* target type determination 
 * we support a single string representing
 *  - an IP address
 *  - a network in CIDR 192.168.123.0/24 notation
 *  - a hostname
 */

int targets(char *arg) {

	char *p;
	char *str;

	int mask;
	struct in_addr net, host;

	FUNCTION(__FILE__, __FUNCTION__);

	str = strdup(arg);
	if (!str) goto trouble;

	if (strstr(str, "/") != NULL) {
		/* we have a slash so assume a.b.c.d/mask notation */
		p = strstr(str, "/");
		if (!p) goto trouble;
		*p = '\0';
		host.s_addr = inet_addr(str);
		if (host.s_addr == INADDR_NONE) {
			fprintf(stderr, "invalid IP address\n");
			goto trouble;
		}
		mask = atoi(p+1);
		if ((mask > 32) || (mask < 1)) {
			fprintf(stderr, "netmask out of bounds\n");
			goto trouble;
		}
		/* determine network address */
		net.s_addr = (host.s_addr) & ~htonl((uint32_t)(pow(2, 32-mask)-1));
		config.net_mask = (unsigned int) mask;
		config.net_base = net;
		config.mode = MODE_NET;
	} else if ((host.s_addr = inet_addr(str)) != INADDR_NONE) {
		/* IP address */
		config.net_ip = host;
		config.mode = MODE_IP;
	} else {
		/* check for hostname */
		struct hostent *hp;

		hp = gethostbyname(str);
		if (hp == NULL) {
			herror("cannot resolve host");
			goto trouble;
		}
		if (hp->h_addrtype != AF_INET) {
			printf("need AF_INET type address\n");
			goto trouble;
		}
		memcpy(&(host.s_addr), hp->h_addr, hp->h_length);
		config.target_host = (char *) arg;
		config.net_ip = host;
		config.mode = MODE_HOST;
	}
	free(str);
	return 1;
trouble:
	if (str) free(str);
	return 0;
}

int main (int argc, char **argv) {

	extern struct config_t config;
	int opt; /* getopt() */

	struct in_addr target_ip;
	unsigned int i;

	/* defaults */
	config.debug = 0;
	config.flags = FLAG_EMPTY | FLAG_UDP_TRANSPORT;
	config.mode = MODE_NONE;
	config.target_host = NULL;
	config.localip = NULL;
	config.net_ip.s_addr = INADDR_NONE;
	config.net_base.s_addr = INADDR_NONE;
	config.net_mask = 0;
	config.sip_port = DEFAULT_PORT;
	config.sip_domain = NULL;
	config.timeout = DEFAULT_TIMEOUT;
	config.requestfile = NULL;

	printf("\nsmap %s <hscholz@raisdorf.net> "
		"http://www.wormulon.net/\n\n", SMAP_VERSION);

	while ((opt = getopt(argc, argv, "hdloOp:w:D:f:P:tu")) != EOF) {
		switch (opt) {
			case 'h': /* help */
				usage(0, argv);
				break;
			case 'd': /* increase debugging */
				config.debug++;
				break;
			case 'o': /* fingerprinting with less output */
				config.flags |= FLAG_FINGERPRINT;
				config.flags |= FLAG_FINGERPRINT_SMALL;
				break;
			case 'O': /* fingerprinting */
				config.flags |= FLAG_FINGERPRINT;
				config.flags &= ~FLAG_FINGERPRINT_SMALL;
				break;
			case 'p': /* SIP destination port */
				config.sip_port = atoi(optarg);
				if (config.sip_port < 0)
					usage(1, argv);
				break;
			case 'w': /* select() timeout in msec */
				config.timeout = atoi(optarg);
				if ((config.timeout < 50) || (config.timeout > 60*1000)) {
					fprintf(stderr, "timeout out of bounds\n");
					usage(1, argv);
				}
				break;
			case 'D': /* SIP domain */
				config.sip_domain = optarg;
				break;
			case 'f': /* file containing custom request */
				config.requestfile = optarg;
				break;
			case 'P': /* nmap-like -P0 */
				if (optarg[0] == '0') {
					config.flags |= FLAG_NOICMPPING;
				}
				break;
			case 'l': /* fingerprinting learning mode */
				/* learning for fingerprinting somehow only makes
				 * sense with fingerprinting also enabled */
				config.flags |= (FLAG_FPLEARNING | FLAG_FINGERPRINT);
				break;
			case 't': /* TCP transport */
				/* UDP is default so TCP would override it */
				config.flags ^= FLAG_UDP_TRANSPORT;
				config.flags |= FLAG_TCP_TRANSPORT;
				break;
			case 'u': /* UDP transport */
				config.flags ^= FLAG_TCP_TRANSPORT;
				config.flags |= FLAG_UDP_TRANSPORT;
				break;
			default:
				usage(1, argv);
				break;
		}
	}
	if (argc >= 2) {
		if (!targets(argv[argc-1])) {
			return 1;
		}
	} else {
		usage(1, argv);
	}

	/* set SIP domain to 'localhost' */
	if (config.sip_domain == NULL)
		config.sip_domain = strdup("localhost");

	/* obtain IP
	 * STUN seems to be more portable than various local methods so
	 * try it first and only fall back to other methods
	 */
	if (!stun_getip("194.97.4.42")) {
		error(ERR_NOTICE,
			"STUN Binding Request failed, falling back to ioctl()");
		if (!get_localip()) {
			error(ERR_FATAL, "could not obtain local IP address");
		}
	}

	/* don't try ICMP ping or raw socket stuff if we don't have root priv. */
	if (geteuid() != 0) {
		config.flags |= FLAG_NOROOT;
	}

	/* initialize statistics storage */
	stats_init();

	/* print debug information */
	if (config.debug >= 2) {
		fprintf(stderr, "smap configuration:\n");
		fprintf(stderr, "  SIP domain: %s\n", config.sip_domain);
		fprintf(stderr, "  SIP port: %d\n", config.sip_port);
		fprintf(stderr, "  timeout: %u msec\n", config.timeout);
		fprintf(stderr, "  fingerprinting: %s\n",
			config.flags & FLAG_FINGERPRINT ? "enabled" : "disabled");
		fprintf(stderr, "  local IP: %s\n\n", config.localip);
	}

	/* load fingerprinting database */
	if (!fingerprint_loaddb()) {
		fprintf(stderr, "cannot load fingerprinting database\n");
		return 0;
	}
#ifdef FP_DUMP
	fingerprint_dumpdb();
	fingerprint_destroydb();
	if (config.localip) free(config.localip);
	exit(0);
#endif

	/* register signal handlers */
	signal(SIGINT, sighandler);

	randomizer_init();

	switch (config.mode) {
		case MODE_IP:
		case MODE_HOST:
			if (scan_host(config.net_ip) == 0)
				goto error_while_scanning;
			break;
		case MODE_NET:
			for (i = 0; i < pow(2, 32-config.net_mask); i++) {
				target_ip.s_addr = config.net_base.s_addr + htonl(i);
				scan_host(target_ip);
			}
			break;
		case MODE_NONE:
		default:	
			usage(1, argv);
			goto error_while_scanning;
			break; /* make gcc happy */
	}
	stats_dump();
error_while_scanning:

	fingerprint_destroydb();
	if (config.localip) free(config.localip);

	return 0;
}
#endif /* _SMAP_C */
